﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using AHD.Emulators_Organizer.Base;
using System.Drawing;
namespace EmulatorsOrganizer
{
    public class TreeNode_Console : TreeNode
    {
        EmuConsole _Console = new EmuConsole();
        public EmuConsole Console
        {
            get { return _Console; }
            set
            {
                _Console = value;
                this.Text = _Console.Name;
                RefreshCategories();
            }
        }
        public void RefreshName()
        {
            this.Text = _Console.Name;
        }
        public void RefreshCategories()
        {
            this.Nodes.Clear();
            foreach (Category Cat in _Console.Categories)
            {
                TreeNode_Category Tr = new TreeNode_Category();
                Tr.CategoryIn = Cat;
                this.Nodes.Add(Tr);
            }
        }
    }
    public class TreeNode_Category : TreeNode
    {
        Category _Category = new Category();
        public Category CategoryIn
        {
            get { return _Category; }
            set
            {
                _Category = value;
                this.Text = value.Name;
            }
        }
        public void RefreshName()
        {
            this.Text = _Category.Name;
        }
    }
    public class ListViewItem_Emulator : ListViewItem
    {
        Emulator _Emu = new Emulator();
        public Emulator Emu
        {
            get { return _Emu; }
            set
            {
                _Emu = value;
                this.Text = _Emu.Name;
            }
        }
        public void RefreshName()
        {
            this.Text = _Emu.Name;
        }
    }
    public class ListViewItem_Rom : ListViewItem
    {
        Rom _Rom = new Rom();
        public Rom ROM
        {
            get { return _Rom; }
            set
            {
                _Rom = value;
                RefreshName();
            }
        }
        public void RefreshName()
        {
            SubItems.Clear();
            SubItems.Add("");//Name
            SubItems.Add("");//_Rom.RomSize
            SubItems.Add("");//rating
            SubItems.Add("");//has image
            SubItems.Add("");//has cover
            SubItems.Add("");//has info
            SubItems.Add("");//has history
            SubItems.Add("");//has manual
            SubItems.Add("");//file type       
            SubItems.Add("");//_Rom.Path       
            SubItems.Add("");//year
            SubItems.Add("");//year
            SubItems.Add("");//year
            this.Text = _Rom.Name;
        }
    }
    public class TreeNode_PlayList : TreeNode
    {
        PlayList _PlayList = new PlayList();
        public PlayList PlayList
        {
            get { return _PlayList; }
            set
            {
                _PlayList = value;
                this.Text = _PlayList.Name;
            }
        }
        public void RefreshName()
        {
            this.Text = _PlayList.Name;
        }
    }
    public class ListViewItem_PlayListItem : ListViewItem
    {
        PlayListItem _PlayListItem = new PlayListItem();
        public PlayListItem PlayListItem
        {
            get { return _PlayListItem; }
            set
            {
                _PlayListItem = value;
                RefreshName();
            }
        }
        public void RefreshName()
        {
            SubItems.Clear();
            SubItems.Add("");//Name
            SubItems.Add("");//_Rom.RomSize
            SubItems.Add("");//rating
            SubItems.Add("");//has image
            SubItems.Add("");//has cover
            SubItems.Add("");//has info
            SubItems.Add("");//has history
            SubItems.Add("");//has manual
            SubItems.Add("");//file type       
            SubItems.Add("");//_Rom.Path
            SubItems.Add("");//year
            SubItems.Add("");//year
            SubItems.Add("");//year
            this.Text = _PlayListItem.Rom.Name;
            //SubItems[0].Text = _PlayListItem.Rom.Name;
            //SubItems[6].Text = _PlayListItem.Rom.Path;
            //SubItems[1].Text = _PlayListItem.Rom.RomSize;
        }
    }

    public class TreeNode_CommandLinesGroup : TreeNode
    {
        CommandLinesGroup _CommandLineGroup = new CommandLinesGroup();
        public CommandLinesGroup CommandLinesGroup
        {
            get { return _CommandLineGroup; }
            set
            {
                _CommandLineGroup = value;
                RefreshName();
            }
        }
        public void RefreshName()
        {
            this.Text = _CommandLineGroup.Name;
        }
        public void RefreshCommandLines(int imageindex, int selectedimageindex)
        {
            Nodes.Clear();
            foreach (CommandLine cm in _CommandLineGroup.CommandLines)
            {
                TreeNode_CommandLine cmm = new TreeNode_CommandLine();
                cmm.CommandLine = cm;
                cmm.ImageIndex = imageindex;
                cmm.SelectedImageIndex = selectedimageindex;
                cmm.Checked = cm.Enabled;
                cmm.RefreshParameters(imageindex, selectedimageindex);

                Nodes.Add(cmm);
            }
        }
    }
    public class TreeNode_CommandLine : TreeNode
    {
        CommandLine _commandline = new CommandLine();
        public CommandLine CommandLine
        {
            get { return _commandline; }
            set
            {
                _commandline = value; RefreshName();
            }
        }
        public void RefreshName()
        {
            this.Text = _commandline.Name + " (" + _commandline.commandLine + ")";
        }
        public void RefreshParameters(int imageindex, int selectedimageindex)
        {
            Nodes.Clear();
            foreach (Parameter pr in _commandline.Parameters)
            {
                TreeNode_Parameter prr = new TreeNode_Parameter();
                prr.Parameter = pr;
                prr.ImageIndex = imageindex;
                prr.SelectedImageIndex = selectedimageindex;
                prr.Checked = pr.Enabled;
                Nodes.Add(prr);
            }
        }
    }
    public class TreeNode_Parameter : TreeNode
    {
        Parameter _Parameter = new Parameter();
        public Parameter Parameter
        {
            get { return _Parameter; }
            set
            {
                _Parameter = value; RefreshName();
            }
        }
        public void RefreshName()
        {
            this.Text = _Parameter.parameter;
        }
    }
    public enum ActiveElement
    {
        ConsoleOrCategory, RomOrPlaylistItem, Playlist, Emulator, Info, History
    }
    public enum SortMode
    {
        None, Name, Size, Rating, Path, PlayedTime, FileType
    }
}
