﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using AHD.Emulators_Organizer.Base;
using System.Threading;
using BER.CDCat.Export;
using IsoCreator;
using System.Diagnostics;
using AHD.Utilities;

namespace EmulatorsOrganizer
{
    public partial class Frm_BuildIso : Form
    {
        Project project;
        Thread m_thread = null;
        Thread ProgressThread = null;
        IsoCreator.IsoCreator m_creator = null;
        public Frm_BuildIso(Project project)
        {
            this.project = project;
            InitializeComponent();
        }

        void PROGRESS()
        {
            #region CHECK
            if (textBox_FolderPath.Text.Length == 0)
            {
                MessageBox.Show("Please browse for a temp folder");
                textBox_FolderPath.SelectAll();
                ProgressThread.Abort();
                return;
            }
            if (!Directory.Exists(textBox_FolderPath.Text))
            {
                MessageBox.Show("This temp folder isn't exist");
                textBox_FolderPath.SelectAll(); ProgressThread.Abort();
                return;
            }
            if (checkBox_WriteIso.Checked)
            {
                if (textBox_IsoPath.Text.Length == 0)
                {
                    MessageBox.Show("Please browse to save the iso file");
                    textBox_IsoPath.SelectAll(); ProgressThread.Abort();
                    return;
                }
                if (textBox_VolumeName.Text.Length == 0)
                {
                    MessageBox.Show("Please enter the volume name of the iso");
                    textBox_VolumeName.SelectAll(); ProgressThread.Abort();
                    return;
                }
            }
            #endregion
            //START THE PROGRESS
            DisableButtons();
            AddLog("Progress started [" + DateTime.Now.ToShortDateString() + " " +
                DateTime.Now.ToShortTimeString() + "]", 0);
            listView_Errors.Items.Clear();
            List<string> ERRORS = new List<string>();
            SetMaximumBar2(5);
            #region 1 Clear the temp folder
            AddLog("Clearing the temp folder ...", 4);
            //Delete all the folders
            string[] files = Directory.GetFiles(textBox_FolderPath.Text, "*", SearchOption.AllDirectories);
            SetStatus2("Clearing from files");
            SetMaximumBar1(files.Length);
            for (int i = 0; i < files.Length; i++)
            {
                try
                {
                    int H = (100 * i) / files.Length;
                    SetStatus1("Removing files " + H.ToString() + "%");
                    File.Delete(files[i]);
                    SetProgress1(i + 1);
                }
                catch
                {
                    EditLog("Clearing the temp folder FAILD !!", 2);
                    AddError("Progress faild, could not clear the temp folder !!", 2);
                    SetMaximumBar1(1);
                    SetMaximumBar2(1);
                    SetProgress1(1);
                    SetProgress2(1);
                    SetStatus1("ABORT");
                    SetStatus2("ABORT");
                    EnableButtons(); return;
                }
            }
            //Delete all the directories
            string[] dirs = Directory.GetDirectories(textBox_FolderPath.Text, "*", SearchOption.TopDirectoryOnly);
            SetStatus2("Clearing from folders");
            SetMaximumBar1(dirs.Length);
            for (int i = 0; i < dirs.Length; i++)
            {
                try
                {
                    int H = (100 * i) / files.Length;
                    SetStatus1("Removing folders " + H.ToString() + "%");
                    Directory.Delete(dirs[i], true);
                    SetProgress1(i + 1);
                }
                catch
                {
                    EditLog("Clearing the temp folder FAILD !!", 2);
                    AddError("Progress faild, could not clear the temp folder !!", 2);
                    SetMaximumBar1(1);
                    SetMaximumBar2(1);
                    SetProgress1(1);
                    SetProgress2(1);
                    SetStatus1("ABORT");
                    SetStatus2("ABORT");
                    EnableButtons(); return;
                }
            }
            //Done , to the next step
            EditLog("Clearing the temp folder done [" + DateTime.Now.ToShortDateString() + " " +
                DateTime.Now.ToShortTimeString() + "]", 1);
            SetProgress2(1);
            #endregion
            #region 2 Create the folders and files
            AddLog("Creating folders and files ...", 4);
            bool founderrors = false;
            Project pro = new Project();
            //if we need to create project, just do it !!
            if (checkBox_CreateProject.Checked)
            {
                pro.Name = project.Name;
            }
            #region Loop through consoles of the project, create the folders for it
            if (project.Consoles.Count == 0)
            {
                EditLog("Creating folders and files FAILD", 2);
                AddError("Progress faild, there are no console in this profile.", 2);
                SetMaximumBar1(1); SetMaximumBar2(1);
                SetProgress1(1);
                SetProgress2(1);
                SetStatus1("ABORT");
                SetStatus2("ABORT");
                EnableButtons(); return;
            }
            else
                foreach (EmuConsole con in project.Consoles)
                {
                    if (con.Roms.Count == 0)
                    {
                        AddError("There is no rom in the console (" + con.Name + "), Console ignored !!", 3);
                        founderrors = true;
                        continue;
                    }
                    EmuConsole NewConsole = new EmuConsole();
                    //1 create the console ..
                    SetStatus2("Creating console (" + con.Name + ")");
                    //create the folder
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name);
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name, 2);
                        AddError("Console ignored (" + con.Name + ")", 3);
                        founderrors = true;
                        continue;
                    }
                    //Create the folders of the console
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\ROMS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\ROMS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\EMULATORS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\EMULATORS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\SNAPSHOTS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\SNAPSHOTS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\COVERS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\COVERS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\TEXTS\\INFOS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\TEXTS\\INFOS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\TEXTS\\HISTORY");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\TEXTS\\HISTORY", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\MANUALS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\MANUALS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\VIDEOS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\VIDEOS", 2);
                    }
                    try
                    {
                        Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\SOUNDS");
                    }
                    catch
                    {
                        AddError("Unable to create directory : " + textBox_FolderPath.Text + "\\" + con.Name + "\\SOUNDS", 2);
                    }
                    //if we have to create project ..
                    if (checkBox_CreateProject.Checked)
                    {
                        NewConsole.Name = con.Name;
                        NewConsole.Categories.AddRange(con.Categories);
                        NewConsole.Extensions = con.Extensions;
                        NewConsole.Icon = con.Icon;
                        NewConsole.InfoFolder = ".\\" + con.Name + "\\TEXTS\\INFOS";
                        NewConsole.HistoryFolder = ".\\" + con.Name + "\\TEXTS\\HISTORY";
                        pro.Consoles.Add(NewConsole);
                    }

                    //2 The Roms
                    SetMaximumBar1(con.Roms.Count);
                    int i = 0;
                    foreach (Rom ro in con.Roms)
                    {
                        if (File.Exists(Utilities.GetFullPath(ro.Path)))
                        {
                            int H = (100 * i) / con.Roms.Count;
                            SetProgress1(i + 1);
                            SetStatus1("Copying roms " + H.ToString() + "%");
                            try
                            {
                                File.Copy(Utilities.GetFullPath(ro.Path),
                                    textBox_FolderPath.Text + "\\" + con.Name + "\\ROMS\\" +
                                    Path.GetFileName(ro.Path), true);
                            }
                            catch
                            {
                                AddError("Unable to copy rom : " + Utilities.GetFullPath(ro.Path), 2);
                            }
                            if (checkBox_CreateProject.Checked)
                            {
                                Rom NewRom = new Rom();
                                NewRom.Name = ro.Name;
                                NewRom.Path = ".\\" + con.Name + "\\ROMS\\" + Path.GetFileName(ro.Path);
                                NewRom.Categories = ro.Categories;
                                NewRom.Links = ro.Links;
                                NewRom.CommandLineGroupsEmulator = ro.CommandLineGroupsEmulator;
                                NewRom.HasSpecialCommandLine = ro.HasSpecialCommandLine;
                                //Copy the snapshots
                                if (ro.Snapshots == null)
                                    ro.Snapshots = new List<string>();
                                NewRom.Snapshots = new List<string>();
                                foreach (string SnapShot in ro.Snapshots)
                                {
                                    if (File.Exists(Utilities.GetFullPath(SnapShot)))
                                    {
                                        try
                                        {
                                            Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\SNAPSHOTS\\"
                                          + Path.GetFileNameWithoutExtension(ro.Path) + "\\");
                                            File.Copy(Utilities.GetFullPath(SnapShot),
                                          textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\SNAPSHOTS\\"
                                          + Path.GetFileNameWithoutExtension(ro.Path) + "\\" +
                                          Path.GetFileName(SnapShot), true);
                                        }
                                        catch
                                        {
                                            AddError("Unable to copy snapshot : " + Utilities.GetFullPath(SnapShot), 3);
                                        }
                                        NewRom.Snapshots.Add(".\\" + con.Name + "\\PICTURES\\SNAPSHOTS\\" +
                                            Path.GetFileNameWithoutExtension(ro.Path) + "\\" + Path.GetFileName(SnapShot));
                                    }
                                    else
                                    {
                                        if (SnapShot.Length > 0)
                                        {
                                            AddError("Snapshot file isn't exist : " + SnapShot +
                                          ", This snapshot ignored.", 3);
                                        }
                                    }
                                }
                                //Copy the covers
                                if (ro.Covers == null)
                                    ro.Covers = new List<string>();
                                NewRom.Covers = new List<string>();
                                foreach (string cover in ro.Covers)
                                {
                                    if (File.Exists(Utilities.GetFullPath(cover)))
                                    {
                                        try
                                        {
                                            Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\COVERS\\"
                                                + Path.GetFileNameWithoutExtension(ro.Path) + "\\");
                                            File.Copy(Utilities.GetFullPath(cover),
                                          textBox_FolderPath.Text + "\\" + con.Name + "\\PICTURES\\COVERS\\" +
                                            Path.GetFileNameWithoutExtension(ro.Path) + "\\" +
                                          Path.GetFileName(cover), true);
                                        }
                                        catch
                                        {
                                            AddError("Unable to copy cover : " + Utilities.GetFullPath(cover), 3);
                                        }
                                        NewRom.Covers.Add(".\\" + con.Name + "\\PICTURES\\COVERS\\" +
                                            Path.GetFileNameWithoutExtension(ro.Path) + "\\" + Path.GetFileName(cover));
                                    }
                                    else
                                    {
                                        if (cover.Length > 0)
                                        {
                                            AddError("Cover file isn't exist : " + cover +
                                          ", This cover ignored.", 3);
                                        }
                                    }
                                }
                                //Copy manual
                                if (File.Exists(ro.PDFPath))
                                {
                                    try
                                    {
                                        File.Copy(Utilities.GetFullPath(ro.PDFPath),
                                      textBox_FolderPath.Text + "\\" + con.Name + "\\MANUALS\\" +
                                      Path.GetFileName(ro.PDFPath), true);
                                    }
                                    catch
                                    {
                                        AddError("Unable to copy manual file : " + Utilities.GetFullPath(ro.PDFPath), 3);
                                    }
                                    NewRom.PDFPath = ".\\" + con.Name + "\\MANUALS\\" + Path.GetFileName(ro.PDFPath);
                                }
                                //Copy the sound files
                                if (ro.SoundRecords == null)
                                    ro.SoundRecords = new List<string>();
                                NewRom.SoundRecords = new List<string>();
                                foreach (string sound in ro.SoundRecords)
                                {
                                    if (File.Exists(Utilities.GetFullPath(sound)))
                                    {
                                        try
                                        {
                                            Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\SOUNDS\\"
                                          + Path.GetFileNameWithoutExtension(ro.Path) + "\\");
                                            File.Copy(Utilities.GetFullPath(sound),
                                          textBox_FolderPath.Text + "\\" + con.Name + "\\SOUNDS\\"
                                          + Path.GetFileNameWithoutExtension(ro.Path) + "\\" +
                                          Path.GetFileName(sound), true);
                                        }
                                        catch
                                        {
                                            AddError("Unable to copy sound file : " + Utilities.GetFullPath(sound), 3);
                                        }
                                        NewRom.SoundRecords.Add(".\\" + con.Name + "\\SOUNDS\\" +
                                            Path.GetFileNameWithoutExtension(ro.Path) + "\\" + Path.GetFileName(sound));
                                    }
                                    else
                                    {
                                        if (sound.Length > 0)
                                        {
                                            AddError("Sound file isn't exist : " + sound +
                                          ", This sound file ignored.", 3);
                                        }
                                    }
                                }
                                //Copy video
                                if (ro.Videos == null)
                                    ro.Videos = new List<string>();
                                NewRom.Videos = new List<string>();
                                foreach (string cover in ro.Videos)
                                {
                                    if (File.Exists(Utilities.GetFullPath(cover)))
                                    {
                                        try
                                        {
                                            Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\VIDEOS\\"
                                                + Path.GetFileNameWithoutExtension(ro.Path) + "\\");
                                            File.Copy(Utilities.GetFullPath(cover),
                                          textBox_FolderPath.Text + "\\" + con.Name + "\\VIDEOS\\" +
                                            Path.GetFileNameWithoutExtension(ro.Path) + "\\" +
                                          Path.GetFileName(cover), true);
                                        }
                                        catch
                                        {
                                            AddError("Unable to copy video : " + Utilities.GetFullPath(cover), 3);
                                        }
                                        NewRom.Videos.Add(".\\" + con.Name + "\\VIDEOS\\" +
                                            Path.GetFileNameWithoutExtension(ro.Path) + "\\" + Path.GetFileName(cover));
                                    }
                                    else
                                    {
                                        if (cover.Length > 0)
                                        {
                                            AddError("Video file isn't exist : " + cover +
                                          ", This video ignored.", 3);
                                        }
                                    }
                                }
                                NewRom.Rating = ro.Rating;
                                NewRom.RomSize = ro.RomSize;
                                NewConsole.Roms.Add(NewRom);
                            }

                        }
                        else
                        {
                            AddError("Rom isn't exist : " + ro.Path +
                              ", This rom ignored from the list.", 3);
                        }
                        i++;
                    }

                    //3 the info text files
                    if (Directory.Exists(con.InfoFolder))
                    {
                        string[] textinfos = Directory.GetFiles(con.InfoFolder);
                        SetMaximumBar1(textinfos.Length);
                        for (i = 0; i < textinfos.Length; i++)
                        {
                            int H = (100 * i) / textinfos.Length; SetProgress1(i);
                            SetStatus1("Copying info text " + H.ToString() + "%");
                            try
                            {
                                File.Copy(textinfos[i],
                                    textBox_FolderPath.Text + "\\" + con.Name + "\\TEXTS\\INFOS\\" +
                                    Path.GetFileName(textinfos[i]));
                            }
                            catch
                            {
                                AddError("Unable to copy info file : " + textinfos[i], 3);
                            }
                        }
                    }
                    //4 HISTORY files
                    if (Directory.Exists(con.HistoryFolder))
                    {
                        string[] textinfos = Directory.GetFiles(con.HistoryFolder);
                        SetMaximumBar1(textinfos.Length);
                        for (i = 0; i < textinfos.Length; i++)
                        {
                            int H = (100 * i) / textinfos.Length; SetProgress1(i);
                            SetStatus1("Copying history text file " + H.ToString() + "%");
                            try
                            {
                                File.Copy(textinfos[i],
                                    textBox_FolderPath.Text + "\\" + con.Name + "\\TEXTS\\HISTORY\\" +
                                    Path.GetFileName(textinfos[i]));
                            }
                            catch
                            {
                                AddError("Unable to copy history text file : " + textinfos[i], 3);
                            }
                        }
                    }
                    //5 Emulators
                    i = 0;
                    if (con.Emulators.Count == 0)
                    {
                        AddError("There is no emulator in the console (" + con.Name + ")", 3);
                        founderrors = true;
                    }
                    else
                        foreach (Emulator emu in con.Emulators)
                        {
                            if (Directory.Exists(Utilities.GetFullPath(emu.Folder)))
                            {
                                //Create the emulator folder
                                Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\EMULATORS\\" + emu.Name);
                                //copy the emulator files
                                string RootFolder = Utilities.GetFullPath(emu.Folder);
                                string[] EmuFiles = Directory.GetFiles(RootFolder, "*", SearchOption.AllDirectories);
                                SetMaximumBar1(EmuFiles.Length);
                                for (int o = 0; o < EmuFiles.Length; o++)
                                {
                                    int H = (100 * o) / EmuFiles.Length;
                                    SetProgress1(o + 1);
                                    SetStatus1("Preparing emulators " + H.ToString() + "%");
                                    string DIR = Path.GetDirectoryName(EmuFiles[o]).Substring(RootFolder.Length) + "\\";
                                    Directory.CreateDirectory(textBox_FolderPath.Text + "\\" + con.Name + "\\EMULATORS\\" + emu.Name + "\\" + DIR);
                                    try
                                    {
                                        File.Copy(EmuFiles[o],
                                           textBox_FolderPath.Text + "\\" + con.Name + "\\EMULATORS\\" + emu.Name + "\\" + DIR +
                                           Path.GetFileName(EmuFiles[o]));
                                    }
                                    catch
                                    {
                                        AddError("Unable to copy emulator's file : " + EmuFiles[o], 3);
                                    }
                                }
                                //If create project ....
                                if (checkBox_CreateProject.Checked)
                                {
                                    Emulator NewEmu = new Emulator();
                                    NewEmu.Name = emu.Name;
                                    NewEmu.Icon = emu.Icon;
                                    NewEmu.CommandLinesGroups = emu.CommandLinesGroups;
                                    NewEmu.Path = ".\\" + con.Name + "\\EMULATORS\\" + emu.Name + "\\" + Path.GetFileName(emu.Path);
                                    NewConsole.Emulators.Add(NewEmu);
                                }
                            }
                            else
                            {
                                AddError("Emulator's folder isn't exist : " + emu.Name +
                                     ", This emulator ignored from the list.", 3);
                            }
                            i++;
                        }
                }
            #endregion
            #region Loop through playlists
            if (checkBox_CreateProject.Checked)
            {
                foreach (PlayList PL in project.PlayLists)
                {
                    PlayList NewPLaylist = new PlayList();
                    SetStatus2("Creating playlist (" + PL.Name + ")");
                    NewPLaylist.Name = PL.Name;
                    NewPLaylist.Icon = PL.Icon;
                    pro.PlayLists.Add(NewPLaylist);
                    int i = 0;
                    SetMaximumBar1(PL.PlayListItems.Count);
                    foreach (PlayListItem item in PL.PlayListItems)
                    {
                        int H = (100 * i) / PL.PlayListItems.Count;
                        SetStatus1("Creating playlist items " + H.ToString() + " %");
                        SetProgress1(i + 1);
                        PlayListItem NewItem = new PlayListItem();
                        //Rom NewROM = new Rom();
                        NewItem.Rom = new Rom();
                        NewItem.Name = item.Name;
                        foreach (EmuConsole newconsole in pro.Consoles)
                        {
                            if (newconsole.Name == item.Console.Name)
                            {
                                NewItem.Console = newconsole;
                                break;
                            }
                        }
                        NewItem.Rom.Name = item.Rom.Name;
                        NewItem.Rom.Categories = item.Rom.Categories;
                        NewItem.Rom.Links = item.Rom.Links;
                        NewItem.Rom.CommandLineGroupsEmulator = item.Rom.CommandLineGroupsEmulator;
                        NewItem.Rom.HasSpecialCommandLine = item.Rom.HasSpecialCommandLine;
                        NewItem.Rom.Rating = item.Rom.Rating;
                        NewItem.Rom.RomSize = item.Rom.RomSize;
                        NewItem.Rom.Path = ".\\" + NewItem.Console.Name + "\\ROMS\\" + Path.GetFileName(item.Rom.Path);
                        if (item.Rom.Snapshots == null)
                            item.Rom.Snapshots = new List<string>();
                        NewItem.Rom.Snapshots = new List<string>();
                        foreach (string snapshot in item.Rom.Snapshots)
                        {
                            if (File.Exists(Utilities.GetFullPath(snapshot)))
                            {
                                NewItem.Rom.Snapshots.Add(".\\" + NewItem.Console.Name + "\\PICTURES\\SNAPSHOTS\\" + Path.GetFileNameWithoutExtension(item.Rom.Path) + "\\" + Path.GetFileName(snapshot));
                            }
                        }
                        //covers
                        if (item.Rom.Covers == null)
                            item.Rom.Covers = new List<string>();
                        NewItem.Rom.Covers = new List<string>();
                        foreach (string cover in item.Rom.Covers)
                        {
                            if (File.Exists(Utilities.GetFullPath(cover)))
                            {
                                NewItem.Rom.Covers.Add(".\\" + NewItem.Console.Name + "\\PICTURES\\COVERS\\" + Path.GetFileNameWithoutExtension(item.Rom.Path) + "\\" + Path.GetFileName(cover));
                            }
                        }
                        //sounds
                        if (item.Rom.SoundRecords == null)
                            item.Rom.SoundRecords = new List<string>();
                        NewItem.Rom.SoundRecords = new List<string>();
                        foreach (string sound in item.Rom.SoundRecords)
                        {
                            if (File.Exists(Utilities.GetFullPath(sound)))
                            {
                                NewItem.Rom.SoundRecords.Add(".\\" + NewItem.Console.Name + "\\SOUNDS\\" + Path.GetFileNameWithoutExtension(item.Rom.Path) + "\\" + Path.GetFileName(sound));
                            }
                        }
                        //Copy manual
                        if (File.Exists(Utilities.GetFullPath(item.Rom.PDFPath)))
                        {
                            NewItem.Rom.PDFPath = ".\\" + NewItem.Console.Name + "\\MANUALS\\" + Path.GetFileName(item.Rom.PDFPath);
                        }
                        //Copy video
                        if (item.Rom.Videos == null)
                            item.Rom.Videos = new List<string>();
                        NewItem.Rom.Videos = new List<string>();
                        foreach (string cover in item.Rom.Videos)
                        {
                            if (File.Exists(Utilities.GetFullPath(cover)))
                            {
                                NewItem.Rom.Videos.Add(".\\" + NewItem.Console.Name + "\\VIDEOS\\" + Path.GetFileNameWithoutExtension(item.Rom.Path) + "\\" + Path.GetFileName(cover));
                            }
                        }
                        NewPLaylist.PlayListItems.Add(NewItem);
                        i++;
                    }
                }
            }
            #endregion
            //Done , to the next step

            if (!founderrors)
                EditLog("Creating folders and files done [" + DateTime.Now.ToShortDateString() + " " +
              DateTime.Now.ToShortTimeString() + "]", 1);
            else
                EditLog("Creating folders and files done [" + DateTime.Now.ToShortDateString() + " " +
             DateTime.Now.ToShortTimeString() + "]", 3);
            SetProgress2(2);
            #endregion
            #region 3 Create Project
            SetProgress2(3);
            SetStatus2("Create project");
            if (checkBox_CreateProject.Checked)
            {
                try
                {
                    AddLog("Creating the project ...", 4);
                    SetMaximumBar1(1);
                    SetStatus1("Saving the project");
                    FileStream fs = new FileStream(textBox_FolderPath.Text + "\\PROJECT.EOP", FileMode.Create);
                    BinaryFormatter formatter = new BinaryFormatter();
                    formatter.Serialize(fs, pro);
                    fs.Close();
                    SetProgress1(1);
                    EditLog("Creating the project done [" + DateTime.Now.ToShortDateString() + " " +
        DateTime.Now.ToShortTimeString() + "]", 1);
                }
                catch
                {
                    EditLog("Creating the project FAILD !! [" + DateTime.Now.ToShortDateString() + " " +
                        DateTime.Now.ToShortTimeString() + "]", 2);
                }
            }
            #endregion
            #region 4 Copy emulators organizer
            SetProgress2(4);
            SetStatus2("Copying Emulators Organizer files");
            if (checkBox_CopyEO.Checked)
            {
                AddLog("Copying Emulators Organizer ...", 4);
                SetMaximumBar1(27);
                bool founerror = false;
                int i = 0;
                try
                {
                    //Copy files
                    try
                    {
                        File.Copy(Application.StartupPath + "\\EmulatorsOrganizer.exe",
                                 textBox_FolderPath.Text + "\\AUTORUN.EXE");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\EmulatorsOrganizer.exe", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\7z.dll",
                                 textBox_FolderPath.Text + "\\7z.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\7z.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Base.dll",
                                 textBox_FolderPath.Text + "\\Base.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Base.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\SevenZipSharp.dll",
                                 textBox_FolderPath.Text + "\\SevenZipSharp.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\SevenZipSharp.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Help.chm",
                                 textBox_FolderPath.Text + "\\Help.chm");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Help.chm", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\7z readme.txt",
                                 textBox_FolderPath.Text + "\\7z readme.txt");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\7z readme.txt", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\License.htm",
                                 textBox_FolderPath.Text + "\\License.htm");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\License.htm", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\AxInterop.AcroPDFLib.dll",
                                 textBox_FolderPath.Text + "\\AxInterop.AcroPDFLib.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\AxInterop.AcroPDFLib.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Controls.dll",
                                 textBox_FolderPath.Text + "\\Controls.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Cntr.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Interop.AcroPDFLib.dll",
                                 textBox_FolderPath.Text + "\\Interop.AcroPDFLib.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Interop.AcroPDFLib.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Utilities.dll",
                                 textBox_FolderPath.Text + "\\Utilities.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Utilities.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Frms.dll",
                                 textBox_FolderPath.Text + "\\Frms.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Frms.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\SharpISO.dll",
                                 textBox_FolderPath.Text + "\\SharpISO.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\SharpISO.dll", 2);
                    }
                    i++;
                    SetProgress1(i);
                    try
                    {
                        File.Copy(Application.StartupPath + "\\AxInterop.WMPLib.dll",
                                 textBox_FolderPath.Text + "\\AxInterop.WMPLib.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\AxInterop.WMPLib.dll", 2);
                    }
                    try
                    {
                        File.Copy(Application.StartupPath + "\\Interop.WMPLib.dll",
                                 textBox_FolderPath.Text + "\\Interop.WMPLib.dll");
                    }
                    catch
                    {
                        AddError("Unable to copy file : " + Application.StartupPath + "\\Interop.WMPLib.dll", 2);
                    }
                    //Create autorun
                    List<string> autorunlines = new List<string>();
                    SetProgress1(i); autorunlines.Add("[autorun]"); i++;
                    SetProgress1(i); autorunlines.Add("open=AUTORUN.EXE"); i++;
                    SetProgress1(i); autorunlines.Add("icon=AUTORUN.EXE,0"); i++;
                    SetProgress1(i);
                    try
                    {
                        File.WriteAllLines(textBox_FolderPath.Text + "\\autorun.inf", autorunlines.ToArray()); i++;
                    }
                    catch
                    {
                        AddError("Unable to save the autorun.inf file !!", 2);
                    }
                    //Settings
                    SetProgress1(i); Settings set = new Settings(); i++;
                    SetProgress1(i); set.RecentProfiles.Clear(); i++;
                    SetProgress1(i); set.RecentProfiles.Add(".\\PROJECT.EOP"); i++;
                    SetProgress1(i); set.RememberLastProject = true; i++;
                    set.RememberSelectedRom = true;
                    //Save settings
                    try
                    {
                        SetProgress1(i); FileStream fs = new FileStream(textBox_FolderPath.Text + "//Settings.st", FileMode.Create, FileAccess.Write); i++;
                        SetProgress1(i); BinaryFormatter formatter = new BinaryFormatter(); i++;
                        SetProgress1(i); formatter.Serialize(fs, set); i++;
                        SetProgress1(i); fs.Close();
                    }
                    catch
                    {
                        AddError("Unable to save settings !!", 2);
                    }
                    if (!founerror)
                        EditLog("Copying Emulators Organizer done [" + DateTime.Now.ToShortDateString() + " " +
DateTime.Now.ToShortTimeString() + "]", 1);
                    else
                        EditLog("Copying Emulators Organizer done [" + DateTime.Now.ToShortDateString() + " " +
DateTime.Now.ToShortTimeString() + "]", 3);
                }
                catch
                {
                    EditLog("Copying Emulators Organizer FAILD !!", 2);
                }
            }
            #endregion
            #region 5 Write ISO
            SetProgress2(5);
            SetStatus2("Writting ISO");
            if (checkBox_WriteIso.Checked)
            {
                AddLog("Writting ISO file", 4);
                m_creator = new IsoCreator.IsoCreator();
                m_creator.Progress += new ProgressDelegate(m_creator_Progress);
                m_creator.Finish += new FinishDelegate(m_creator_Finish);
                m_thread = new Thread(new ParameterizedThreadStart(m_creator.Folder2Iso));
                m_thread.Start(new IsoCreator.IsoCreator.IsoCreatorFolderArgs(textBox_FolderPath.Text, textBox_IsoPath.Text, textBox_VolumeName.Text));
            }
            else
            { FINISH(); }
            #endregion
        }
        void FINISH()
        {
            AddLog("Finishing progress ...", 4);
            SetMaximumBar2(1);
            SetProgress2(1);
            SetStatus2("Done");
            #region Clear the temp
            if (checkBox_DeleteTheTemp.Checked)
            {
                string[] files = Directory.GetFiles(textBox_FolderPath.Text, "*", SearchOption.AllDirectories);
                SetMaximumBar1(files.Length);
                for (int i = 0; i < files.Length; i++)
                {
                    try
                    {
                        int H = (100 * i) / files.Length;
                        SetStatus1("Removing files " + H.ToString() + "%");
                        File.Delete(files[i]);
                        SetProgress1(i);
                    }
                    catch
                    {

                    }
                }
                //Delete all the directories
                string[] dirs = Directory.GetDirectories(textBox_FolderPath.Text, "*", SearchOption.TopDirectoryOnly);
                SetMaximumBar1(dirs.Length);
                for (int i = 0; i < dirs.Length; i++)
                {
                    try
                    {
                        int H = (100 * i) / files.Length;
                        SetStatus1("Removing folders " + H.ToString() + " %");
                        Directory.Delete(dirs[i], true);
                        SetProgress1(i);
                    }
                    catch
                    {

                    }
                }
                //Delete the temp folder
                try
                {
                    SetStatus1("Deleting the temp folder");
                    Directory.Delete(textBox_FolderPath.Text, true);
                }
                catch { }
            }
            #endregion
            if (checkBox_BrowseTheTemp.Checked & !checkBox_DeleteTheTemp.Checked)
            {
                try
                {
                    Process.Start(textBox_FolderPath.Text);
                }
                catch { }
            }
            if (checkBox_OpenTheIso.Checked & checkBox_WriteIso.Checked)
            {
                try
                {
                    Process.Start(textBox_IsoPath.Text);
                }
                catch { }
            }
            SetMaximumBar1(1);
            SetProgress1(1);
            SetStatus1("Done");
            EditLog("All progress finished at : " + DateTime.Now.ToShortDateString() + " " +
  DateTime.Now.ToShortTimeString(), 1);
            EnableButtons();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                MessageBox.Show("The temp folder will be EMPTYED, be careful !!", "WARNING", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                textBox_FolderPath.Text = fol.SelectedPath;
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            SaveFileDialog sav = new SaveFileDialog();
            sav.Filter = "ISO Image file(*.iso)|*.iso";
            if (sav.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                textBox_IsoPath.Text = sav.FileName;
            }
        }
        //START THE PROGRESS
        private void button3_Click(object sender, EventArgs e)
        {
            ProgressThread = new Thread(new ThreadStart(PROGRESS));
            ProgressThread.Start();
        }

        void m_creator_Finish(object sender, FinishEventArgs e)
        {
            EditLog("Write ISO done at : " + " " + DateTime.Now.ToShortDateString() + " " +
DateTime.Now.ToShortTimeString(), 1);
            ProgressThread = new Thread(new ThreadStart(FINISH));
            ProgressThread.Start();
        }
        void m_creator_Progress(object sender, ProgressEventArgs e)
        {
            if (e.Action != null)
            {
                if (!this.InvokeRequired)
                {
                    this.SetLabelStatus(e.Action);
                }
                else
                {
                    this.Invoke(new SetLabelDelegate(SetLabelStatus), e.Action);
                }
            }

            if (e.Maximum != -1)
            {
                if (!this.InvokeRequired)
                {
                    this.SetProgressMaximum(e.Maximum);
                }
                else
                {
                    this.Invoke(new SetNumericValueDelegate(SetProgressMaximum), e.Maximum);
                }
            }

            if (!this.InvokeRequired)
            {
                progressBar1.Value = (e.Current <= progressBar1.Maximum) ? e.Current : progressBar1.Maximum;
            }
            else
            {
                int value = (e.Current <= progressBar1.Maximum) ? e.Current : progressBar1.Maximum;
                this.Invoke(new SetNumericValueDelegate(SetProgressValue), value);
            }
        }

        private void checkBox_WriteIso_CheckedChanged(object sender, EventArgs e)
        {
            textBox_VolumeName.Enabled = checkBox_WriteIso.Checked;
            button2.Enabled = checkBox_WriteIso.Checked;
            textBox_IsoPath.Enabled = checkBox_WriteIso.Checked;
            label1.Enabled = checkBox_WriteIso.Checked;
        }
        private void listView1_SizeChanged(object sender, EventArgs e)
        {
            columnHeader1.Width = listView_Log.Width;
        }
        private void listView1_Validated(object sender, EventArgs e)
        {
            columnHeader1.Width = listView_Log.Width;
        }

        void SetStatus1(string text)
        {
            if (!this.InvokeRequired)
            {
                this.SetLabelStatus(text);
            }
            else
            {
                this.Invoke(new SetLabelDelegate(SetLabelStatus), text);
            }
        }
        void SetStatus2(string text)
        {
            if (!this.InvokeRequired)
            {
                this.SetLabelStatus1(text);
            }
            else
            {
                this.Invoke(new SetLabelDelegate(SetLabelStatus1), text);
            }
        }
        void SetProgress1(int value)
        {
            if (!this.InvokeRequired)
            {
                progressBar1.Value = value;
            }
            else
            {
                this.Invoke(new SetNumericValueDelegate(SetProgressValue), value);
            }
        }
        void SetProgress2(int value)
        {
            if (!this.InvokeRequired)
            {
                progressBar2.Value = value;
            }
            else
            {
                this.Invoke(new SetNumericValueDelegate(SetProgressValue2), value);
            }
        }
        void SetMaximumBar1(int value)
        {
            if (value != -1)
            {
                if (!this.InvokeRequired)
                {
                    this.SetProgressMaximum(value);
                }
                else
                {
                    this.Invoke(new SetNumericValueDelegate(SetProgressMaximum), value);
                }
            }
        }
        void SetMaximumBar2(int value)
        {
            if (value != -1)
            {
                if (!this.InvokeRequired)
                {
                    this.SetProgressMaximum2(value);
                }
                else
                {
                    this.Invoke(new SetNumericValueDelegate(SetProgressMaximum2), value);
                }
            }
        }
        void AddLog(string text, int ImageIndex)
        {
            if (!this.InvokeRequired)
            {
                this.AddLogItem(text, ImageIndex);
            }
            else
            {
                this.Invoke(new AddLogItemDelegate(AddLogItem), new object[] { text, ImageIndex });
            }
        }
        void EditLog(string text, int ImageIndex)
        {
            if (!this.InvokeRequired)
            {
                this.EditLogItem(text, ImageIndex);
            }
            else
            {
                this.Invoke(new AddLogItemDelegate(EditLogItem), new object[] { text, ImageIndex });
            }
        }
        void AddError(string text, int ImageIndex)
        {
            if (!this.InvokeRequired)
            {
                this.AddErrorItem(text, ImageIndex);
            }
            else
            {
                this.Invoke(new AddLogItemDelegate(AddErrorItem), new object[] { text, ImageIndex });
            }
        }
        void EnableButtons()
        {
            if (!this.InvokeRequired)
            {
                EnableButtonsAndGr();
            }
            else
            {
                this.Invoke(new EnableDisable(EnableButtonsAndGr));
            }
        }
        void DisableButtons()
        {
            if (!this.InvokeRequired)
            {
                DisableButtonsAndGr();
            }
            else
            {
                this.Invoke(new EnableDisable(DisableButtonsAndGr));
            }
        }

        private delegate void SetLabelDelegate(string text);
        private delegate void SetNumericValueDelegate(int value);
        private delegate void AddLogItemDelegate(string text, int ImageIndex);
        private delegate void EnableDisable();

        private void EnableButtonsAndGr()
        {
            groupBox2.Enabled = true;
            button3.Enabled = true;
            textBox3.Enabled = true;
            groupBox7.Enabled = true;
            groupBox8.Enabled = true;
            button6.Enabled = true;
        }
        private void DisableButtonsAndGr()
        {
            groupBox2.Enabled = false;
            button3.Enabled = false;
            textBox3.Enabled = false;
            groupBox7.Enabled = false;
            groupBox8.Enabled = false;
            button6.Enabled = false;
        }
        private void SetLabelStatus(string text)
        {
            this.Statuslabel1.Text = text;
            this.Statuslabel1.Refresh();
        }
        private void SetLabelStatus1(string text)
        {
            this.Statuslabel2.Text = text;
            this.Statuslabel2.Refresh();
        }
        private void SetProgressValue(int value)
        {
            this.progressBar1.Value = value;
        }
        private void SetProgressValue2(int value)
        {
            this.progressBar2.Value = value;
        }
        private void SetProgressMaximum(int maximum)
        {
            this.progressBar1.Maximum = maximum;
        }
        private void SetProgressMaximum2(int maximum)
        {
            this.progressBar2.Maximum = maximum;
        }
        private void AddLogItem(string text, int ImageIndex)
        {
            listView_Log.Items.Add(text, ImageIndex);
            listView_Log.Items[listView_Log.Items.Count - 1].EnsureVisible();
        }
        private void AddErrorItem(string text, int ImageIndex)
        {
            listView_Errors.Items.Add(text, ImageIndex);
            listView_Errors.Items[listView_Errors.Items.Count - 1].EnsureVisible();
        }
        private void EditLogItem(string text, int ImageIndex)
        {
            listView_Log.Items[listView_Log.Items.Count - 1].Text = text;
            listView_Log.Items[listView_Log.Items.Count - 1].ImageIndex = ImageIndex;
        }

        //The iso progress
        private void button6_Click(object sender, EventArgs e)
        {
            #region CHECK
            if (textBox1.Text.Length == 0)
            {
                MessageBox.Show("Please browse for a temp folder");
                textBox1.SelectAll();
                return;
            }
            if (!Directory.Exists(textBox1.Text))
            {
                MessageBox.Show("This folder isn't exist !!");
                textBox1.SelectAll();
                return;
            }
            if (textBox2.Text.Length == 0)
            {
                MessageBox.Show("please browse to save the iso file");
                textBox2.SelectAll();
                return;
            }
            if (textBox3.Text.Length == 0)
            {
                MessageBox.Show("Please enter the iso volume name");
                textBox3.SelectAll();
                return;
            }
            #endregion
            DisableButtons();
            listView_Log.Items.Add("Progress started [" + DateTime.Now.ToShortDateString() + " " +
                DateTime.Now.ToShortTimeString() + "]", 0);
            listView_Log.Refresh();
            listView_Log.Items.Add("Writing iso file ....", 4);
            listView_Log.Refresh();
            SetStatus2("Writing iso");
            m_creator = new IsoCreator.IsoCreator();
            m_creator.Progress += new ProgressDelegate(m_creator_Progress);
            m_creator.Finish += new FinishDelegate(m_creator_Finish1);
            m_thread = new Thread(new ParameterizedThreadStart(m_creator.Folder2Iso));
            m_thread.Start(new IsoCreator.IsoCreator.IsoCreatorFolderArgs(textBox1.Text, textBox2.Text, textBox3.Text));

        }
        void m_creator_Finish1(object sender, FinishEventArgs e)
        {
            listView_Log.Items[listView_Log.Items.Count - 1].Text = ("Write iso, done at : " + DateTime.Now.ToShortDateString() + " " +
DateTime.Now.ToShortTimeString());
            listView_Log.Items[listView_Log.Items.Count - 1].ImageIndex = 1;
            listView_Log.Refresh();
            progressBar1.Maximum = 1;
            progressBar1.Value = 1;
            Statuslabel1.Text = "Done";
            listView_Log.Items.Add("Finishing progress ...", 4);
            listView_Log.Refresh();
            progressBar2.Maximum = 1;
            progressBar2.Value = 1;
            Statuslabel2.Text = "Done";
            listView_Log.Items[listView_Log.Items.Count - 1].Text = ("Progress finished successfully [" + DateTime.Now.ToShortDateString() + " " +
DateTime.Now.ToShortTimeString() + "]");
            listView_Log.Items[listView_Log.Items.Count - 1].ImageIndex = 1;
            listView_Log.Refresh();
            EnableButtons();
        }
        private void button4_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                textBox1.Text = fol.SelectedPath;
            }
        }
        private void button5_Click(object sender, EventArgs e)
        {
            SaveFileDialog sav = new SaveFileDialog();
            sav.Filter = "ISO Image file(*.iso)|*.iso";
            if (sav.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                textBox2.Text = sav.FileName;
            }
        }
        private void listView2_SizeChanged(object sender, EventArgs e)
        {
            columnHeader2.Width = listView_Errors.Width;
        }
        private void listView2_Validated(object sender, EventArgs e)
        {
            columnHeader2.Width = listView_Errors.Width;
        }

        private void button7_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void Frm_BuildIso_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (m_thread != null)
            {
                if (m_thread.IsAlive)
                {
                    if (MessageBox.Show("Are you sure, there is a progress running ?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.No)
                    {
                        e.Cancel = true;
                    }
                    else
                    {
                        m_thread.Abort();
                    }
                }
            }
            else if (ProgressThread != null)
            {
                if (ProgressThread.IsAlive)
                {
                    if (MessageBox.Show("Are you sure, there is a progress running ?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.No)
                    {
                        e.Cancel = true;
                    }
                    else
                    { ProgressThread.Abort(); }
                }
            }
        }
    }
}
