﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using AHD.Utilities;
using EmulatorsOrganizer.Frms;

namespace EmulatorsOrganizer
{
    public partial class Frm_Options : Form
    {
        public Frm_Options()
        {
            InitializeComponent();
            checkBox1_autominimize.Checked = Program.UserSettings.AutoMinimize;
            checkBox2_rememberlastopened.Checked = Program.Setting.RememberLastProject;
            checkBox3_rememberlastselected.Checked = Program.Setting.RememberSelectedRom;
            checkBox4_extractfirst.Checked = Program.UserSettings.ExtractArchiveFirst;
            checkBox5_Cleartemp.Checked = Program.UserSettings.ClearTemp;
            checkBox6_usesnapinstatus.Checked = Program.UserSettings.UseSnapInStatus;
            checkBox1_forcestretchsnapshot.Checked = Program.UserSettings.forceStretchSnapshots;
            checkBox2_stretchcovers.Checked = Program.UserSettings.forceStretchCovers;
            checkBox1_autoSave.Checked = Program.UserSettings.AutoSave;
            textBox1.Text = Program.UserSettings.TempFolder;
            if (Program.UserSettings.ClearTempFolderExceptExtensions == null)
                Program.UserSettings.ClearTempFolderExceptExtensions = new System.Collections.Specialized.StringCollection();
            foreach (string ex in Program.UserSettings.ClearTempFolderExceptExtensions)
                comboBox1.Items.Add(ex);
            switch (Program.UserSettings.GoogleSearchMode)
            {
                case GoogleSearchMode.RomName: comboBox2_google.SelectedIndex = 0; break;
                case GoogleSearchMode.RomName_Console: comboBox2_google.SelectedIndex = 1; break;
                case GoogleSearchMode.RomName_Console_Emulator: comboBox2_google.SelectedIndex = 2; break;
            }
            textBox2_googleperfix.Text = Program.UserSettings.GoogleSearchPerfix;
        }

        private void button6_Click(object sender, EventArgs e)
        {
            checkBox1_autominimize.Checked = true;
            checkBox2_rememberlastopened.Checked = true;
            checkBox3_rememberlastselected.Checked = true;
            checkBox4_extractfirst.Checked = true;
            checkBox5_Cleartemp.Checked = true;
            checkBox6_usesnapinstatus.Checked = true;
            checkBox1_forcestretchsnapshot.Checked = true;
            checkBox2_stretchcovers.Checked = true;
            checkBox1_autoSave.Checked = false;
            textBox1.Text = ".\\Temp";
            comboBox1.Items.Clear();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Program.UserSettings.AutoMinimize = checkBox1_autominimize.Checked;
            Program.Setting.RememberLastProject = checkBox2_rememberlastopened.Checked;
            Program.Setting.RememberSelectedRom = checkBox3_rememberlastselected.Checked;
            Program.UserSettings.ExtractArchiveFirst = checkBox4_extractfirst.Checked;
            Program.UserSettings.ClearTemp = checkBox5_Cleartemp.Checked;
            Program.UserSettings.UseSnapInStatus = checkBox6_usesnapinstatus.Checked;
            Program.UserSettings.forceStretchSnapshots = checkBox1_forcestretchsnapshot.Checked;
            Program.UserSettings.forceStretchCovers = checkBox2_stretchcovers.Checked;
            Program.UserSettings.TempFolder = textBox1.Text;
            Program.UserSettings.AutoSave = checkBox1_autoSave.Checked;
            Program.UserSettings.ClearTempFolderExceptExtensions = new System.Collections.Specialized.StringCollection();
            foreach (string ex in comboBox1.Items)
                Program.UserSettings.ClearTempFolderExceptExtensions.Add(ex);
            Program.UserSettings.GoogleSearchPerfix = textBox2_googleperfix.Text;
            switch (comboBox2_google.SelectedIndex)
            {
                case 0: Program.UserSettings.GoogleSearchMode = GoogleSearchMode.RomName; break;
                case 1: Program.UserSettings.GoogleSearchMode = GoogleSearchMode.RomName_Console; break;
                case 2: Program.UserSettings.GoogleSearchMode = GoogleSearchMode.RomName_Console_Emulator; break;
            }
            Program.UserSettings.Save();
            Program.SaveSettings();
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fo = new FolderBrowserDialog();
            fo.SelectedPath = Utilities.GetFullPath(textBox1.Text);
            if (fo.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                textBox1.Text = Utilities.GetDotPath(fo.SelectedPath);
        }

        private void button5_Click(object sender, EventArgs e)
        {
            Frm_EnterName name = new Frm_EnterName("Enter extension", "");
            if (name.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                int i = 0;
                foreach (string ex in comboBox1.Items)
                {
                    if (ex.ToLower() == "." + name.NameEntered.ToLower())
                    {
                        comboBox1.SelectedIndex = i;
                        return;
                    }
                    i++;
                }
                comboBox1.Items.Add("." + name.NameEntered.ToLower());
                comboBox1.SelectedIndex = comboBox1.Items.Count - 1;
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            if (comboBox1.Items.Count > 0)
            {
                comboBox1.Items.RemoveAt(comboBox1.SelectedIndex);
            }
            if (comboBox1.Items.Count > 0)
                comboBox1.SelectedIndex = 0;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            Close();
        }
    }
}
