﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using AHD.Emulators_Organizer.Base;
using AHD.Utilities;
using System.IO;

namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_AddRoms : Form
    {
        string _lastSelected = "";
        EmuConsole console;
        public Frm_AddRoms(EmuConsole console)
        {
            this.console = console;
            InitializeComponent();
            if (console.Folders == null)
                console.Folders = new List<string>();
            foreach (string dir in console.Folders)
            {
                if (Directory.Exists(Utilities.GetFullPath(dir)))
                    listBox1.Items.Add(dir);
            }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            fol.SelectedPath = _lastSelected;
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                _lastSelected = fol.SelectedPath;
                foreach (string path in listBox1.Items)
                {
                    if (path.ToLower() == Utilities.GetDotPath(fol.SelectedPath).ToLower())
                        return;
                }
            }
            listBox1.Items.Add(Utilities.GetDotPath(fol.SelectedPath));
        }

        private void linkLabel2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (listBox1.SelectedIndex >= 0)
                listBox1.Items.RemoveAt(listBox1.SelectedIndex);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            Close();
        }
        //start ...
        private void button1_Click(object sender, EventArgs e)
        {
            //1 get dirs
            if (listBox1.Items.Count == 0)
            {
                MessageBox.Show("There's no folder !!");
                return;
            }
            label1.Visible = progressBar1.Visible = true;
            List<string> dirs = new List<string>();
            console.Folders.Clear();
            foreach (string folder in listBox1.Items)
            {
                console.Folders.Add(folder);
                if (checkBox1_subfolder.Checked)
                {
                    dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder), "*", SearchOption.AllDirectories));
                }
                else
                {
                    dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder)));
                }
            }
            //2 search the dirs for roms
            if (checkBox2_clearold.Checked)
                console.Roms.Clear();
            for (int i = 0; i < dirs.Count; i++)
            {
                foreach (string ext in console.Extensions)
                {
                    //get the rom with the same type
                    if (Path.GetExtension(dirs[i]).ToLower() == ext.ToLower())
                    {
                        //replace same rom file
                        if (checkBox3_replace.Checked)
                        {
                            for (int j = 0; j < console.Roms.Count; j++)
                            {
                                if (Utilities.GetFullPath(console.Roms[j].Path).ToLower() == dirs[i].ToLower())
                                {
                                    console.Roms.RemoveAt(j);
                                    j = -1;
                                }
                            }
                        }
                        //add new rom
                        Rom rom = new Rom();
                        rom.Name = Path.GetFileNameWithoutExtension(dirs[i]);
                        rom.Path = Utilities.GetDotPath(dirs[i]);
                        rom.RomSize = Utilities.GetFileSize(dirs[i]);
                        console.Roms.Add(rom);
                        int x = (i * 100) / dirs.Count;
                        progressBar1.Value = x;
                        label1.Refresh();
                        progressBar1.Refresh();
                    }
                }
            }
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            OpenFileDialog fil = new OpenFileDialog();
            fil.Title = "Add roms";
            fil.Multiselect = true;
            string filter = console.Name + " roms (";
            if (console.Extensions.Count == 1)
            { filter += "*" + console.Extensions[0]; }
            else
            {
                foreach (string ext in console.Extensions)
                {
                    filter += "*" + ext;
                    if (console.Extensions[console.Extensions.Count - 1] != ext)
                    { filter += ","; }
                }
            }
            filter += ")|";
            foreach (string ext in console.Extensions)
            {
                filter += "*" + ext + ";";
            }
            fil.Filter = filter;
            if (fil.ShowDialog(this) == DialogResult.OK)
            {
                bool Replace = checkBox3_replace.Checked;
                if (checkBox2_clearold.Checked)
                    console.Roms.Clear();
                foreach (string R in fil.FileNames)
                {
                    foreach (string ext in console.Extensions)
                    {
                        if (ext.ToLower() == Path.GetExtension(R).ToLower())
                        {
                            Rom ro = new Rom();
                            ro.Name = Path.GetFileNameWithoutExtension(R);
                            ro.Path = Utilities.GetDotPath(R);
                            ro.RomSize = Utilities.GetFileSize(R);
                            if (Replace)
                            {
                                for (int g = 0; g < console.Roms.Count; g++)
                                {
                                    if (ro.Path == console.Roms[g].Path)
                                    { console.Roms.RemoveAt(g); g = -1; }
                                }
                            }
                            console.Roms.Add(ro);
                            break;
                        }
                    }
                }
                DialogResult = System.Windows.Forms.DialogResult.OK;
                this.Close();
            }
        }
    }
}
