﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_CompressTool : Form
    {
        public Frm_CompressTool()
        {
            InitializeComponent();
            comboBox_ArchiveFormat.SelectedIndex = 0;
            comboBox_CopressinLevel.SelectedItem = "Normal";
        }
        Thread ProgressThread;
        string _ArchiveExtension = "";
        SevenZip.OutArchiveFormat _ArchiveFormat = SevenZip.OutArchiveFormat.SevenZip;
        SevenZip.CompressionLevel _CompressionLevel = SevenZip.CompressionLevel.Fast;
        private delegate void EnableDisable();
        private delegate void SetLabelDelegate(string text);
        private delegate void SetLogDelegate(string text, Color status);
        private delegate void SetNumericValueDelegate(int value);
        void DisableButtons()
        {
            if (!this.InvokeRequired)
            {
                DisableButtonsAndGr();
            }
            else
            {
                this.Invoke(new EnableDisable(DisableButtonsAndGr));
            }
        }
        void EnableButtons()
        {
            if (!this.InvokeRequired)
            {
                EnableButtonsAndGr();
            }
            else
            {
                this.Invoke(new EnableDisable(EnableButtonsAndGr));
            }
        }
        void WriteLog(string text, Color status)
        {
            if (!this.InvokeRequired)
                WriteLog1(text,  status);
            else
                this.Invoke(new SetLogDelegate(WriteLog1), new object[] { text, status });
        }
        void WriteLog1(string text, Color status)
        {
            richTextBox1.SelectionStart = richTextBox1.Text.Length;
            richTextBox1.SelectionColor = status;
            richTextBox1.SelectedText = text + "\n";
            richTextBox1.SelectionColor = Color.Black;
            richTextBox1.ScrollToCaret();
        }
        void SetStatus(string text)
        {
            if (!this.InvokeRequired)
            {
                this.SetLabelStatus(text);
            }
            else
            {
                this.Invoke(new SetLabelDelegate(SetLabelStatus), text);
            }
        }
        void SetStatus1(string text)
        {
            if (!this.InvokeRequired)
            {
                this.SetLabelStatus1(text);
            }
            else
            {
                this.Invoke(new SetLabelDelegate(SetLabelStatus1), text);
            }
        }
        void SetMaximumBar(int value)
        {
            if (value != -1)
            {
                if (!this.InvokeRequired)
                {
                    this.SetProgressMaximum(value);
                }
                else
                {
                    this.Invoke(new SetNumericValueDelegate(SetProgressMaximum), value);
                }
            }
        }
        void SetProgress(int value)
        {
            if (!this.InvokeRequired)
            {
                progressBar1.Value = value;
            }
            else
            {
                this.Invoke(new SetNumericValueDelegate(SetProgressValue), value);
            }
        }
        void SetMaximumBar1(int value)
        {
            if (value != -1)
            {
                if (!this.InvokeRequired)
                {
                    this.SetProgressMaximum1(value);
                }
                else
                {
                    this.Invoke(new SetNumericValueDelegate(SetProgressMaximum1), value);
                }
            }
        }
        void SetProgress1(int value)
        {
            if (!this.InvokeRequired)
            {
                progressBar2.Value = value;
            }
            else
            {
                this.Invoke(new SetNumericValueDelegate(SetProgressValue1), value);
            }
        }
        private void EnableButtonsAndGr()
        {
            button3.Enabled = groupBox1.Enabled = groupBox2.Enabled = groupBox3.Enabled = true;
            button5.Text = "&Start";
        }
        private void DisableButtonsAndGr()
        {
            button3.Enabled = groupBox1.Enabled = groupBox2.Enabled = groupBox3.Enabled = false;
            button5.Text = "&Abort";
        }
        private void SetLabelStatus(string text)
        {
            statuslabel.Text = text;
        }
        private void SetLabelStatus1(string text)
        {
            StatusLabel_Compress.Text = text;
        }
        private void SetProgressValue(int value)
        {
            this.progressBar1.Value = value;
        }
        private void SetProgressMaximum(int maximum)
        {
            this.progressBar1.Maximum = maximum;
        }
        private void SetProgressValue1(int value)
        {
            this.progressBar2.Value = value;
        }
        private void SetProgressMaximum1(int maximum)
        {
            this.progressBar2.Maximum = maximum;
        }

        void COMPRESSPROGRESS()
        {
            SevenZip.SevenZipCompressor comper = new SevenZip.SevenZipCompressor();
            comper.ArchiveFormat = _ArchiveFormat;
            comper.CompressionLevel = _CompressionLevel;
            comper.Compressing += new EventHandler<SevenZip.ProgressEventArgs>(comper_Compressing);
            DisableButtons();
            string[] files = Directory.GetFiles(textBox_InputFolder.Text);
            WriteLog(files.Length + " File(s) found...", Color.Black);
            WriteLog("Extract is Starting ...", Color.Green);
            string[] dirs = Directory.GetDirectories(textBox_InputFolder.Text);
            int compresscount = 0;
            SetMaximumBar(files.Length);
            for (int i = 0; i < files.Length; i++)
            {
                try
                {
                    SetProgress(i + 1);
                    int Precento = (100 * (i + 1)) / files.Length;
                    SetStatus("Compressing files " + (i + 1) + " of " + files.Length + " (" + Precento + "%)");
                    if (!checkBox_Encrypt.Checked)
                        comper.CompressFiles(textBox_OutputFolder.Text + "\\" + Path.GetFileNameWithoutExtension(files[i]) + _ArchiveExtension, new string[] { files[i] });
                    else
                        comper.CompressFilesEncrypted(textBox_OutputFolder.Text + "\\" + Path.GetFileNameWithoutExtension(files[i]) + _ArchiveExtension, textBox_Password.Text, new string[] { files[i] });
                    compresscount++;
                }
                catch (Exception ex)
                {
                    WriteLog("x Faild to compress : " + Path.GetFileName(files[i]) + ", exception: " + ex.Message, Color.Red);
                }
            }
            SetMaximumBar(dirs.Length);
            for (int i = 0; i < dirs.Length; i++)
            {
                try
                {
                    SetProgress(i + 1);
                    int Precento = (100 * (i + 1)) / dirs.Length;
                    SetStatus("Compressing folders " + (i + 1) + " of " + dirs.Length + " (" + Precento + "%)");
                    if (!checkBox_Encrypt.Checked)
                    {
                        comper.PreserveDirectoryRoot = checkBox_IncludeParentFolder.Checked;
                        comper.CompressDirectory(dirs[i], textBox_OutputFolder.Text + "\\" + Path.GetFileNameWithoutExtension(dirs[i]) + _ArchiveExtension);
                    }
                    else
                    {
                        comper.PreserveDirectoryRoot = checkBox_IncludeParentFolder.Checked;
                        comper.CompressDirectory(dirs[i], textBox_OutputFolder.Text + "\\" + Path.GetFileNameWithoutExtension(dirs[i]) + _ArchiveExtension, textBox_Password.Text);
                    }
                    compresscount++;
                }
                catch (Exception ex)
                {
                    WriteLog("x Faild to compress : " + Path.GetFileName(files[i]) + ", exception: " + ex.Message, Color.Red);
                }
            }
            SetStatus("Finished");
            SetStatus1("Finished");
            WriteLog("Finished, " + compresscount + " file(s) compressed.", Color.Red);
            EnableButtons();
        }
        void EXTRACTPROGRESS()
        {
            DisableButtons();
            string[] files = Directory.GetFiles(textBox_InputFolder.Text);
            int extractedcount = 0;
            WriteLog(files.Length+" File(s) found...", Color.Black);
            WriteLog("Extract is Starting ...", Color.Green);
            SetMaximumBar(files.Length);
            for (int i = 0; i < files.Length; i++)
            {
                SetProgress(i + 1);
                int Precento = (100 * (i + 1)) / files.Length;
                SetStatus("Extracting " + (i + 1) + " of " + files.Length + " (" + Precento + "%)");
                SevenZip.SevenZipExtractor ext;
                try
                {
                    if (checkBox_extractpassword.Checked)
                        ext = new SevenZip.SevenZipExtractor(files[i], textBox_Extractpassword.Text);
                    else
                        ext = new SevenZip.SevenZipExtractor(files[i]);

                    ext.Extracting += new EventHandler<SevenZip.ProgressEventArgs>(ext_Extracting);
                    if (checkBox_ExtractFolders.Checked)
                    {
                        Directory.CreateDirectory(textBox_OutputFolder.Text + "\\" + Path.GetFileNameWithoutExtension(files[i]));
                        ext.ExtractArchive(textBox_OutputFolder.Text + "\\" + Path.GetFileNameWithoutExtension(files[i]));
                        extractedcount++;
                    }
                    else
                    {
                        ext.ExtractArchive(textBox_OutputFolder.Text);
                        extractedcount++;
                    }
                }
                catch (Exception ex)
                {
                    WriteLog("x Faild to extract : " + Path.GetFileName(files[i]) + ", exception: " + ex.Message, Color.Red);
                }
            }
            SetStatus("Finished");
            SetStatus1("Finished");
            WriteLog("Finished, " + extractedcount + " file(s) extracted.", Color.Red);
            EnableButtons();
        }

        void ext_Extracting(object sender, SevenZip.ProgressEventArgs e)
        {
            SetProgress1(e.PercentDone);
            SetStatus1("Extracting " + e.PercentDone + "%");
        }

        void comper_Compressing(object sender, SevenZip.ProgressEventArgs e)
        {
            SetProgress1(e.PercentDone);
            SetStatus1("Compressing " + e.PercentDone + "%");
        }
        private void comboBox_ArchiveFormat_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBox_ArchiveFormat.SelectedItem.ToString())
            {
                case "7z": _ArchiveFormat = SevenZip.OutArchiveFormat.SevenZip; break;
                case "Zip": _ArchiveFormat = SevenZip.OutArchiveFormat.Zip; break;
                case "BZip2": _ArchiveFormat = SevenZip.OutArchiveFormat.BZip2; break;
                case "GZip": _ArchiveFormat = SevenZip.OutArchiveFormat.GZip; break;
                case "Tar": _ArchiveFormat = SevenZip.OutArchiveFormat.Tar; break;
                case "XZ": _ArchiveFormat = SevenZip.OutArchiveFormat.XZ; break;
            }
        }
        private void comboBox_CopressinLevel_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBox_CopressinLevel.SelectedItem.ToString())
            {
                case "Fast": _CompressionLevel = SevenZip.CompressionLevel.Fast; break;
                case "High": _CompressionLevel = SevenZip.CompressionLevel.High; break;
                case "Low": _CompressionLevel = SevenZip.CompressionLevel.Low; break;
                case "None": _CompressionLevel = SevenZip.CompressionLevel.None; break;
                case "Normal": _CompressionLevel = SevenZip.CompressionLevel.Normal; break;
                case "Ultra": _CompressionLevel = SevenZip.CompressionLevel.Ultra; break;
            }
        }
        //Start
        private void button5_Click(object sender, EventArgs e)
        {
            //Checks
            if (textBox_InputFolder.Text.Length == 0)
            {
                MessageBox.Show("Please browse for the input folder");
                return;
            }
            if (textBox_OutputFolder.Text.Length == 0)
            {
                MessageBox.Show("Please browse for the output folder");
                return;
            }
            if (checkBox_Encrypt.Checked)
            {
                if (textBox_Password.Text.Length == 0)
                {
                    MessageBox.Show("Please enter a password for archive encryption");
                    return;
                }
            }
            if (checkBox_extractpassword.Checked)
            {
                if (textBox_Extractpassword.Text.Length == 0)
                {
                    MessageBox.Show("Please enter a password for encrypted archives extraction");
                    return;
                }
            }
            if (ProgressThread != null)
            {
                if (ProgressThread.IsAlive)
                {
                    if (MessageBox.Show("Are you sure ?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        ProgressThread.Abort();
                        SetProgress(0);
                        SetProgress1(0);
                        SetStatus("Aborted by user");
                        SetStatus1("Aborted by user");
                        EnableButtons();
                    }
                    return;
                }
            }
            _ArchiveExtension = "." + comboBox_ArchiveFormat.SelectedItem.ToString().ToLower();
            if (radioButton_Compress.Checked)
            {
                ProgressThread = new Thread(new ThreadStart(COMPRESSPROGRESS));
                ProgressThread.Start();
            }
            else
            {
                ProgressThread = new Thread(new ThreadStart(EXTRACTPROGRESS));
                ProgressThread.Start();
            }
        }
        private void radioButton_Compress_CheckedChanged(object sender, EventArgs e)
        {
            groupBox4.Enabled = radioButton_Compress.Checked;
            groupBox6.Enabled = !radioButton_Compress.Checked;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                textBox_InputFolder.Text = fol.SelectedPath;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fol = new FolderBrowserDialog();
            if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                textBox_OutputFolder.Text = fol.SelectedPath;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void checkBox_extractpassword_CheckedChanged(object sender, EventArgs e)
        {
            textBox_Extractpassword.Enabled = checkBox_extractpassword.Checked;
        }

        private void Frm_CompressTool_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (ProgressThread != null)
            {
                if (ProgressThread.IsAlive)
                {
                    if (MessageBox.Show("Are you sure ?", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        ProgressThread.Abort();
                        SetProgress(0);
                        SetProgress1(0);
                        SetStatus("Aborted by user");
                        SetStatus1("Aborted by user");
                        EnableButtons();
                    }
                    else
                        e.Cancel = true;
                }
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            Help.ShowHelp(this, ".\\Help.chm", HelpNavigator.KeywordIndex, "Compression tool");
        }

        private void button6_Click(object sender, EventArgs e)
        {
            richTextBox1.Copy();
        }
    }
}