﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Threading;
using AHD.Utilities;

namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_RomDownload : Form
    {
        string url, DestinationFile = "";
        Thread mainThread; 
        WebClient client;
        bool _Ok = false;
        public bool OK
        { get { return _Ok; } }
        delegate void SetIntValue(int value);
        delegate void SetStringValue(string Value);
        delegate void DoVerb();
        void SetProgress(int value)
        {
            try
            {
                if (!this.InvokeRequired)
                    SetProgress1(value);
                else
                    this.Invoke(new SetIntValue(SetProgress1), new object[] { value });
            }
            catch { }
        }
        void SetProgress1(int value)
        {
            progressBar1.Value = value;
        }
        void SetStatus(string value)
        {
            try
            {
                if (!this.InvokeRequired)
                    SetStatus1(value);
                else
                    this.Invoke(new SetStringValue(SetStatus1), new object[] { value });
            }
            catch { }
        }
        void SetStatus1(string value)
        {
            label1.Text = value;
        }
        void CloseSuccess()
        {
            if (!this.InvokeRequired)
                CloseSuccess1();
            else
                this.Invoke(new DoVerb(CloseSuccess1));
        }
        void CloseSuccess1()
        {
            _Ok = true;
            Close();
        }
        void CloseError()
        {
            try
            {
                if (!this.InvokeRequired)
                    CloseError1();
                else
                    this.Invoke(new DoVerb(CloseError));
            }
            catch { }
        }
        void CloseError1()
        {
            _Ok = false;
            Close();
        }
        public Frm_RomDownload(string url, string DestinationFile)
        {
            this.url = url;
            this.DestinationFile = DestinationFile;
            InitializeComponent();
            mainThread = new Thread(new ThreadStart(PROCESS));
            mainThread.Start();
        }
        void PROCESS()
        {
            try
            {
                client = new WebClient();
                client.DownloadProgressChanged += new DownloadProgressChangedEventHandler(client_DownloadProgressChanged);
                client.DownloadFileCompleted += new AsyncCompletedEventHandler(client_DownloadFileCompleted);
                Uri uri = new Uri(url);
                client.DownloadFileAsync(uri, DestinationFile);
                //CloseSuccess();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Can't download file, exception:\n" + ex.Message);
                CloseError();
            }
        }

        void client_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
        {
            CloseSuccess();
        }

        void client_DownloadProgressChanged(object sender, DownloadProgressChangedEventArgs e)
        {
            SetProgress(e.ProgressPercentage);
            string status = e.ProgressPercentage + "% , " + Utilities.GetSize(e.BytesReceived) + " of " + Utilities.GetSize(e.TotalBytesToReceive);
            SetStatus(status);
        }

        private void Frm_RomDownload_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (mainThread != null)
                if (mainThread.IsAlive)
                {
                    client.Dispose();
                    mainThread.Abort();
                    mainThread = null;
                }
            DialogResult = System.Windows.Forms.DialogResult.Cancel;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
