﻿/*  
     This file is part of Emulators Organizer
    A program that organize emulators and roms.
   
    Copyright (C) 2009 - 2012  Ali Hadid, programmed by Ala Hadid

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using AHD.Emulators_Organizer.Base;
using AHD.Utilities;

namespace EmulatorsOrganizer.Frms
{
    public partial class Frm_AddConsoleWizard : Form
    {
        string _lastSelected = "";
        int STEP = 0;
        EmuConsole _console = new EmuConsole();
        public EmuConsole Console
        { get { return _console; } }
        Project _project = new Project();
        public Frm_AddConsoleWizard(Project project)
        {
            _project = project;
            InitializeComponent();
        }
        void RefreshStepItems()
        {
            if (STEP == 0)
            {
                label3.Image = Properties.Resources.cpu;
                label1.Text = "Console Name\n\n" + "SETP1 : Enter the name of the console. (e.g. Nes)";
                label2.Visible = textBox1.Visible = true;
                checkBox1.Visible = button4_Add.Visible = button5_remove.Visible = listBox1.Visible = false;
                button1.Enabled = true;//next 
                button1.Text = "&Next >";
                button2.Enabled = false;//back
            }
            else if (STEP == 1)
            {
                label3.Image = Properties.Resources.File_unkown;
                label1.Text = "Extensions\n\n" + "SETP2 : Add rom file extensions of this console (e.g. .nes for Nes)";
                checkBox1.Visible = label2.Visible = textBox1.Visible = false;
                button4_Add.Visible = button5_remove.Visible = listBox1.Visible = true;
                button1.Enabled = true;//next 
                button1.Text = "&Next >";
                button2.Enabled = true;//back
                RefreshExtensions();
            }
            else if (STEP == 2)
            {
                label3.Image = Properties.Resources.ClosedFolder;
                label1.Text = "Rom Folders\n\n" + "SETP3 : Add folders which contains the rom files for this console.";
                label2.Visible = textBox1.Visible = false;
                checkBox1.Visible = button4_Add.Visible = button5_remove.Visible = listBox1.Visible = true;
                button1.Enabled = true;//next 
                button1.Text = "&Next >";
                button2.Enabled = true;//back
                RefreshFolders();
            }
            else if (STEP == 3)
            {
                label3.Image = Properties.Resources.imgfolde;
                label1.Text = "Snapshots\n\n" + "SETP4 : Add folders which contains the snapshots for roms of this console.";
                checkBox1.Visible = label2.Visible = textBox1.Visible = false;
                button4_Add.Visible = button5_remove.Visible = listBox1.Visible = true;
                button1.Enabled = true;//next 
                button1.Text = "&Next >";
                button2.Enabled = true;//back
                RefreshSnapFolders();
            }
            else if (STEP == 4)
            {
                label3.Image = Properties.Resources.imgfolde;
                label1.Text = "Covers\n\n" + "SETP5 : Add folders which contains the covers for roms of this console.";
                checkBox1.Visible = label2.Visible = textBox1.Visible = false;
                button4_Add.Visible = button5_remove.Visible = listBox1.Visible = true;
                button1.Enabled = true;//next 
                button1.Text = "&Next >";
                button2.Enabled = true;//back
                RefreshCoverFolders();
            }
            else if (STEP == 5)
            {
                label3.Image = Properties.Resources.File_PDF;
                label1.Text = "Manuals\n\n" + "SETP6 : Add folders which contains the manuals for roms of this console.";
                checkBox1.Visible = label2.Visible = textBox1.Visible = false;
                button4_Add.Visible = button5_remove.Visible = listBox1.Visible = true;
                button1.Enabled = true;//next 
                button1.Text = "&Finish";
                button2.Enabled = true;//back
                RefreshManualsFolders();
            }
        }
        void RefreshExtensions()
        {
            //refresh extensions
            listBox1.Items.Clear();
            foreach (string ex in _console.Extensions)
                listBox1.Items.Add(ex);
        }
        void RefreshFolders()
        {
            //refresh folders
            listBox1.Items.Clear();
            foreach (string ex in _console.Folders)
                listBox1.Items.Add(ex);
        }
        void RefreshSnapFolders()
        {
            //refresh folders
            listBox1.Items.Clear();
            foreach (string ex in _console.ImageDirs)
                listBox1.Items.Add(ex);
        }
        void RefreshCoverFolders()
        {
            //refresh folders
            listBox1.Items.Clear();
            foreach (string ex in _console.CoverDirs)
                listBox1.Items.Add(ex);
        }
        void RefreshManualsFolders()
        {
            //refresh folders
            listBox1.Items.Clear();
            foreach (string ex in _console.PDFDirs)
                listBox1.Items.Add(ex);
        }
        void Finish()
        {
            //1 Add roms
            label4.Visible = progressBar1.Visible = true;
            label4.Text = "Adding roms...";
            label4.Refresh();
            progressBar1.Maximum = 100;
            List<string> dirs = new List<string>();
            foreach (string folder in _console.Folders)
            {
                if (checkBox1.Checked)
                {
                    dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder), "*", SearchOption.AllDirectories));
                }
                else
                {
                    dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder)));
                }
            }
            for (int i = 0; i < dirs.Count; i++)
            {
                foreach (string ext in _console.Extensions)
                {
                    //get the rom with the same type
                    if (Path.GetExtension(dirs[i]).ToLower() == ext.ToLower())
                    {
                        //add new rom
                        Rom rom = new Rom();
                        rom.Name = Path.GetFileNameWithoutExtension(dirs[i]);
                        rom.Path = Utilities.GetDotPath(dirs[i]);
                        rom.RomSize = Utilities.GetFileSize(dirs[i]);
                        _console.Roms.Add(rom);
                        int x = (i * 100) / dirs.Count;
                        progressBar1.Value = x;
                        label1.Refresh();
                        progressBar1.Refresh();
                    }
                }
            }
            //2 detect snapshots
            label4.Text = "Detecting snapshots..."; label4.Refresh();
            dirs = new List<string>();
            foreach (string folder in _console.ImageDirs)
            {
                dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder), "*", SearchOption.AllDirectories));
            }
            progressBar1.Maximum = _console.Roms.Count;
            string[] Exs = { ".jpg", ".png", ".bmp", ".gif", ".jpeg", ".tiff", ".tga" };
            foreach (Rom ro in _console.Roms)
            {
                for (int i = 0; i < dirs.Count; i++)
                {
                    bool Found = false;
                    string ImageName = Path.GetFileNameWithoutExtension(dirs[i]).ToLower();
                    string RomRealName = Path.GetFileNameWithoutExtension(ro.Path).ToLower();
                    string ImageExtension = Path.GetExtension(dirs[i]).ToLower();
                    if (ImageName.Length >= RomRealName.Length)
                    {
                        foreach (string ext in Exs)
                        {
                            if (ImageExtension.ToLower() == ext)
                            {
                                if (ImageName.Substring(0, RomRealName.Length).ToLower() == RomRealName.ToLower())
                                {
                                    ro.Snapshots = new List<string>();
                                    ro.Snapshots.Add(dirs[i]);
                                    ro.Snapshots[0] = Utilities.GetDotPath(dirs[i]);

                                    dirs.RemoveAt(i);
                                    i = -1;
                                    Found = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (Found)
                        break;
                }
                try { progressBar1.Value++; }
                catch { }
            }
            //3 detect covers
            label4.Text = "Detecting covers...";
            dirs = new List<string>(); label4.Refresh();
            foreach (string folder in _console.CoverDirs)
            {
                dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder), "*", SearchOption.AllDirectories));
            }
            progressBar1.Maximum = _console.Roms.Count;
            progressBar1.Value = 0;
            Exs = new string[] { ".jpg", ".png", ".bmp", ".gif", ".jpeg", ".tiff", ".tga" };
            foreach (Rom ro in _console.Roms)
            {
                for (int i = 0; i < dirs.Count; i++)
                {
                    bool Found = false;
                    string ImageName = Path.GetFileNameWithoutExtension(dirs[i]).ToLower();
                    string RomRealName = Path.GetFileNameWithoutExtension(ro.Path).ToLower();
                    string ImageExtension = Path.GetExtension(dirs[i]).ToLower();
                    if (ImageName.Length >= RomRealName.Length)
                    {
                        foreach (string ext in Exs)
                        {
                            if (ImageExtension.ToLower() == ext)
                            {
                                if (ImageName.Substring(0, RomRealName.Length).ToLower() == RomRealName.ToLower())
                                {
                                    ro.Covers = new List<string>();
                                    ro.Covers.Add(dirs[i]);
                                    ro.Covers[0] = Utilities.GetDotPath(dirs[i]);

                                    dirs.RemoveAt(i);
                                    i = -1;
                                    Found = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (Found)
                        break;
                }
                try { progressBar1.Value++; }
                catch { }
            }
            //4 detect manuals
            label4.Text = "Detecting manuals..."; label4.Refresh();
            progressBar1.Maximum = _console.Roms.Count; progressBar1.Value = 0;
            Exs = new string[] { ".pdf" };
            dirs = new List<string>(); 
            foreach (string folder in _console.PDFDirs)
            {
                dirs.AddRange(Directory.GetFiles(Utilities.GetFullPath(folder), "*", SearchOption.AllDirectories));
            }
            foreach (Rom ro in _console.Roms)
            {
                for (int i = 0; i < dirs.Count; i++)
                {
                    bool Found = false;
                    string ImageName = Path.GetFileNameWithoutExtension(dirs[i]).ToLower();
                    string RomRealName = Path.GetFileNameWithoutExtension(ro.Path).ToLower();
                    string ImageExtension = Path.GetExtension(dirs[i]).ToLower();
                    if (ImageName.Length >= RomRealName.Length)
                    {
                        foreach (string ext in Exs)
                        {
                            if (ImageExtension.ToLower() == ext)
                            {
                                if (ImageName.Substring(0, RomRealName.Length).ToLower() == RomRealName.ToLower())
                                {
                                    ro.PDFPath = Utilities.GetDotPath(dirs[i]);
                                    dirs.RemoveAt(i);
                                    i = -1;
                                    Found = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (Found)
                        break;
                }
                try { progressBar1.Value++; }
                catch { }
            }

            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            Close();
        }
        //next
        private void button1_Click(object sender, EventArgs e)
        {
            switch (STEP)
            {
                case 0:
                    if (textBox1.Text.Length == 0)
                    {
                        MessageBox.Show("Please enter a name for the console ?!");
                        textBox1.SelectAll();
                        return;
                    }
                    foreach (EmuConsole console in _project.Consoles)
                    {
                        if (console.Name.ToLower() == textBox1.Text.ToLower())
                        {
                            MessageBox.Show("Name already taken by another console");
                            return;
                        }
                    }
                    break;
                case 1:
                    if (listBox1.Items.Count == 0)
                    {
                        MessageBox.Show("Please add extension ?!");
                        return;
                    }
                    break;
                case 2:
                    if (listBox1.Items.Count == 0)
                    {
                        MessageBox.Show("Please add roms folder ?!");
                        return;
                    }
                    break;
            }
            if (STEP < 5)
                STEP++;
            else
            {
                Finish(); return;
            }
            RefreshStepItems();
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            _console.Name = textBox1.Text;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            Close();
        }

        private void button4_Add_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fol;
            switch (STEP)
            {
                case 1:
                    Frm_EnterName nam = new Frm_EnterName("Enter extension", "");
                    if (nam.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        string name = "";
                        if (nam.NameEntered.Substring(0, 1) != ".")
                            name = "." + nam.NameEntered;
                        else
                            name = nam.NameEntered;
                        foreach (string ex in _console.Extensions)
                        {
                            if (ex.ToLower() == name.ToLower())
                                return;
                        }
                        _console.Extensions.Add(name);
                        RefreshExtensions();
                    }
                    break;
                case 2:
                    fol = new FolderBrowserDialog();
                    fol.SelectedPath = _lastSelected;
                    if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        _lastSelected = fol.SelectedPath;
                        foreach (string path in listBox1.Items)
                        {
                            if (path.ToLower() == Utilities.GetDotPath(fol.SelectedPath).ToLower())
                                return;
                        }
                        _console.Folders.Add(Utilities.GetDotPath(fol.SelectedPath));
                        RefreshFolders();
                    }

                    break;
                case 3:
                    fol = new FolderBrowserDialog();
                    fol.SelectedPath = _lastSelected;
                    if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        _lastSelected = fol.SelectedPath;
                        foreach (string path in listBox1.Items)
                        {
                            if (path.ToLower() == Utilities.GetDotPath(fol.SelectedPath).ToLower())
                                return;
                        }
                        _console.ImageDirs.Add(Utilities.GetDotPath(fol.SelectedPath));
                        RefreshSnapFolders();
                    }

                    break;
                case 4:
                    fol = new FolderBrowserDialog();
                    fol.SelectedPath = _lastSelected;
                    if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        _lastSelected = fol.SelectedPath;
                        foreach (string path in listBox1.Items)
                        {
                            if (path.ToLower() == Utilities.GetDotPath(fol.SelectedPath).ToLower())
                                return;
                        }
                        _console.CoverDirs.Add(Utilities.GetDotPath(fol.SelectedPath));
                        RefreshCoverFolders();
                    }

                    break;
                case 5:
                    fol = new FolderBrowserDialog();
                    fol.SelectedPath = _lastSelected;
                    if (fol.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                    {
                        _lastSelected = fol.SelectedPath;
                        foreach (string path in listBox1.Items)
                        {
                            if (path.ToLower() == Utilities.GetDotPath(fol.SelectedPath).ToLower())
                                return;
                        }
                        _console.PDFDirs.Add(Utilities.GetDotPath(fol.SelectedPath));
                        RefreshManualsFolders();
                    }
                    break;
            }
        }

        private void button5_remove_Click(object sender, EventArgs e)
        {
            switch (STEP)
            {
                case 1:
                    if (listBox1.SelectedIndex >= 0)
                    {
                        _console.Extensions.RemoveAt(listBox1.SelectedIndex);
                        RefreshExtensions();
                    }
                    break;
                case 2:
                    if (listBox1.SelectedIndex >= 0)
                    {
                        _console.Folders.RemoveAt(listBox1.SelectedIndex);
                        RefreshFolders();
                    }
                    break;
                case 3:
                    if (listBox1.SelectedIndex >= 0)
                    {
                        _console.ImageDirs.RemoveAt(listBox1.SelectedIndex);
                        RefreshSnapFolders();
                    }
                    break;
                case 4:
                    if (listBox1.SelectedIndex >= 0)
                    {
                        _console.CoverDirs.RemoveAt(listBox1.SelectedIndex);
                        RefreshCoverFolders();
                    }
                    break;
                case 5:
                    if (listBox1.SelectedIndex >= 0)
                    {
                        _console.PDFDirs.RemoveAt(listBox1.SelectedIndex);
                        RefreshManualsFolders();
                    }
                    break;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            STEP--;
            RefreshStepItems();
        }
    }
}
